namespace GruntsWinnerRoom
{
    /*
     * -------------------------------------------------------------------------
     * Constants & enumerators
     * -------------------------------------------------------------------------
     */

    const float         LOOP_INTERVAL       = 0.1f;
    const In_Buttons    USE_BIND            = IN_RELOAD;
    const string        FILTER_TARGETNAME   = "winner";
    const string        ACTIVATE_TARGET     = "tpwinner";



    /*
     * -------------------------------------------------------------------------
     * Variables
     * -------------------------------------------------------------------------
     */

    /** @var CScheduledFunction g_pSchedule Loop. */
    CScheduledFunction@ g_pLoop;



    /*
     * -------------------------------------------------------------------------
     * Functions
     * -------------------------------------------------------------------------
     */

    /**
     * Start thinking.
     * @return void
     */
    void Start()
    {
        if (g_pLoop !is null) {
            g_Game.AlertMessage(at_warning, "[GruntsWinnerRoom] Already thinking.\n");
            return;
        }

        @g_pLoop = g_Scheduler.SetInterval("Think", LOOP_INTERVAL, g_Scheduler.REPEAT_INFINITE_TIMES);
    }

    /**
     * Stop thinking.
     * @return void
     */
    void Stop()
    {
        if (g_pLoop is null) {
            g_Game.AlertMessage(at_warning, "[GruntsWinnerRoom] Not yet thinking.\n");
            return;
        }

        g_Scheduler.RemoveTimer(g_pLoop);
        @g_pLoop = null;
    }

    /**
     * Think.
     * @return void
     */
    void Think()
    {
        for (int i = 1; i <= g_Engine.maxClients; i++) {
            CBasePlayer@ pPlayer = g_PlayerFuncs.FindPlayerByIndex(i);
            if (pPlayer is null or !pPlayer.IsPlayer() or !pPlayer.IsConnected()) {
                continue;
            }

            if (pPlayer.pev.targetname != FILTER_TARGETNAME) {
                continue;
            }

            if ((pPlayer.pev.button & USE_BIND) != IN_RELOAD) {
                continue;
            }

            g_EntityFuncs.FireTargets(ACTIVATE_TARGET, pPlayer, pPlayer, USE_TOGGLE);
        }
    }

}